VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Vector"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'============================================'
' clsVector v1.13 | _void | 11.10.2003 03:32 '
'============================================'

Option Explicit

'---Private constants---'

'-- ERR_INDEXOUTOFBOUNDS = Subscript out of range --'

'{
Private Const ERR_INDEXOUTOFBOUNDS As Long = 9
'}

'---Private variables---'

'-- m_Components = The dynamic array in which all components are saved --'
'-- m_Increment  = The amount by which the capacity of the vector is
'                  incremented when its size becomes greater than its capacity --'
'-- m_Size       = The number of valid components in the vector --'
'-- m_Capacity   = The capacity of the vector --'

'{
Private m_Components() As Variant
Private m_Increment    As Long
Private m_Size         As Long
Private m_Capacity     As Long
'}

Private Sub Class_Initialize()
'---Initializes variables---'

'{
    m_Increment = 0
    m_Size = 0
    m_Capacity = 0
'}
End Sub

Private Sub Class_Terminate()
'---Frees the memory used by the dynamic array and other variables---'

'{
    Erase m_Components
    
    m_Increment = 0
    m_Size = 0
    m_Capacity = 0
'}
End Sub

'---Public properties---'

'-- Capacity[Get]          = Returns the current capacity of the vector --'
'-- Capacity[Let]          = Increases the vectors capacity to ensure
'                            that it can hold at least the specified number
'                            of components --'
'-- CapacityIncrement[Get] = Returns the current increment value --'
'-- CapacityIncrement[Let] = Sets the increment to the specified value that
'                            must be greater than or equal to zero --'
'-- Count[Get]             = Returns the number of components in the vector --'
'-- Count[Let]             = Sets the size of the vector; if the new size
'                            is less than the current size, all components
'                            at this index and greater are discarded --'
'-- Item[Get]              = Returns the component at the specified index --'
'-- Item[Let]              = Sets the component at the specified index
'                            of the vector to the specified item; the previous
'                            component at that position is discarded --'

'{
Public Property Get Capacity() As Long
'{
    Capacity = m_Capacity
'}
End Property

Public Property Let Capacity(ByVal Value As Long)

'-- Value = The minimum capacity --'

'{
    '-Make sure the new capacity is greater than the size
    ' of the vector, we do not want to lose any data-'
    If (m_Size > Value) Then Exit Property
    
    m_Capacity = Value
    
    '-Resize the array-'
    If (m_Capacity = 0) Then
        Erase m_Components
    Else
        ReDim Preserve m_Components(m_Capacity - 1) As Variant
    End If
'}
End Property

Public Property Get CapacityIncrement() As Long
'{
    CapacityIncrement = m_Increment
'}
End Property

Public Property Let CapacityIncrement(ByVal Value As Long)

'-- Value = The new increment value --'

'{
    '-The new value must not be less than zero-'
    If (Value < 0) Then Exit Property
    
    m_Increment = Value
'}
End Property

Public Property Get Count() As Long
'{
    Count = m_Size
'}
End Property

Public Property Let Count(ByVal Value As Long)

'-- Value = The new size of the vector --'

'{
    '-Define variabled-'
    Dim c As Long
    
    '-The new size must be less than the current size, because we do
    ' not want conflicts with the vectors capacity-'
    If (Value > m_Size) Then Exit Property
    
    '-Discard all components greater than index 'value',
    ' the capacity remains untouched-'
    For c = Value To m_Size - 1
        m_Components(c) = Empty
    Next c
    
    '-Set the new size-'
    m_Size = Value
'}
End Property

Public Property Get Item(ByVal Index As Long) As Variant
Attribute Item.VB_UserMemId = 0

'-- Index = An index identifing an item of the vector.
'           'Index' must be a value greater than or equal to zero
'           and less than the current size of the vector --'

'{
    '-The user is not allowed to access reserved items-'
    If (Index >= m_Size) Then
    
        Err.Raise ERR_INDEXOUTOFBOUNDS
        Exit Property
    End If
    
    '-If the component is an object, we have to assign its reference-'
    If IsObject(m_Components(Index)) Then
        Set Item = m_Components(Index)
    Else
        Item = m_Components(Index)
    End If
'}
End Property

Public Property Let Item(ByVal Index As Long, ByRef Value As Variant)

'-- Index = The specified index --'
'-- Value = What the component is to be set to --'

'{
    '-The user is not allowed to access reserved items-'
    If (Index >= m_Size) Then
    
        Err.Raise ERR_INDEXOUTOFBOUNDS
        Exit Property
    End If
    
    '-If the item is an object, we have to assign its reference-'
    If IsObject(Value) Then
        Set m_Components(Index) = Value
    Else
        m_Components(Index) = Value
    End If
'}
End Property
'}

'---Public methods---'

'-- Add         = Adds the specified item to the end of the vector --'
'-- AddArray    = Adds the specified item array to the end of the vector --'
'-- Clear       = Removes all components from the vector --'
'-- Find        = Searches for the first occurence of the given item,
'                 beginning the search at the specified index --'
'-- FindLast    = Searches backwards for the specified item starting from
'                 the specified index --'
'-- Insert      = Inserts the specified item at the specified index --'
'-- InsertArray = Inserts the given item array to the specified position --'
'-- Items       = Returns an enumeration of the vectors valid components --'
'-- Remove      = Deletes the component at the specified index --'
'-- ToString    = Returns a string representation of the vector --'
'-- TrimToSize  = Trims the capacity of the vector to its current
'                 size to minimize the storage of the vector --'

'{
Public Sub Add(ByRef Item As Variant)

'-- Item = the component to be added --'

'{
    '-Increase the size of the vector by one-'
    m_Size = m_Size + 1
    
    '-If there is no free capacity it needs to grow-'
    Do While (m_Capacity < m_Size)
        f_IncreaseCapacity
    Loop
    
    '-If the item is an object, we have to assign its reference-'
    If IsObject(Item) Then
        Set m_Components(m_Size - 1) = Item
    Else
        m_Components(m_Size - 1) = Item
    End If
'}
End Sub

Public Sub AddArray(ByRef Items As Variant)

'-- Items     = the component array to be added --'

'{
    'Define variables-'
    Dim c As Long
    Dim s As Long
    
    s = m_Size
    
    '-Increase the size of the vector by number of components in the array-'
    m_Size = m_Size + UBound(Items) - LBound(Items) + 1
    
    '-If there is no free capacity it needs to grow-'
    Do While (m_Capacity < m_Size)
        f_IncreaseCapacity
    Loop
    
    For c = LBound(Items) To UBound(Items)
    
        '-If the item is an object, we have to assign its reference-'
        If IsObject(Items(c)) Then
            Set m_Components(s) = Items(c)
        Else
            m_Components(s) = Items(c)
        End If
        
        s = s + 1
    Next c
'}
End Sub

Public Sub Clear()
'{
    '-Set the size to zero-'
    m_Size = 0
    
    '-Set each component to empty but do not change the capacity-'
    If (m_Capacity = 0) Then
        Erase m_Components
    Else
        ReDim m_Components(m_Capacity - 1) As Variant
    End If
'}
End Sub

Public Function Find(ByRef Item As Variant, Optional ByVal Index As Long = 0) As Long

'-- Item  = An item to search for --'
'-- Index = The index to start searching from;
'           if not specified, the first index is used--'

'{
    '-Define variables-'
    Dim c As Long
    Dim m As Boolean
    
    'Exit if there are no components-'
    If (m_Size = 0) Then
    
        Find = -1
        Exit Function
    End If
    
    '-The user is not allowed to access reserved items-'
    If (Index >= m_Size) Then
    
        Err.Raise ERR_INDEXOUTOFBOUNDS
        Exit Function
    End If
    
    m = False
    '-If the vector has components...-'
    If (m_Size > 0) Then
    
        '-...test each for equality with the specified item-'
        For c = Index To m_Size - 1
        
            '-If the component is an object, we have
            ' to compare the references-'
            If IsObject(m_Components(c)) Then
                If (m_Components(c) Is Item) Then
                
                    m = True
                    Exit For
                End If
            Else
                If (CStr(m_Components(c)) = CStr(Item)) Then
                
                    m = True
                    Exit For
                End If
            End If
        Next c
    End If
    
    '-Return the index of the first occurrence
    ' or -1 if the item is not found-'
    Find = IIf(m, c, -1)
'}
End Function

Public Function FindLast(ByRef Item As Variant, Optional ByVal Index As Long = -1) As Long

'-- Item  = The item to search for --'
'-- Index = The index to start searching from;
'           if not specified, the last index is used--'

'{
    '-Define variables-'
    Dim c As Long
    Dim m As Boolean
    
    'Exit if there are no components-'
    If (m_Size = 0) Then
    
        FindLast = -1
        Exit Function
    End If
    
    '-If no index is specified the search starts from the last component-'
    If (Index = -1) Then
        Index = m_Size - 1
    End If
    
    '-The user is not allowed to access reserved items-'
    If (Index >= m_Size) Then
    
        Err.Raise ERR_INDEXOUTOFBOUNDS
        Exit Function
    End If
    
    m = False
    '-If the vector has components...-'
    If (m_Size > 0) Then
    
        '-...test each for equality with the specified item-'
        For c = Index To 0 Step -1
        
            '-If the component is an object, we have
            ' to compare the references-'
            If IsObject(m_Components(c)) Then
                If (m_Components(c) Is Item) Then
                
                    m = True
                    Exit For
                End If
            Else
                If (CStr(m_Components(c)) = CStr(Item)) Then
                
                    m = True
                    Exit For
                End If
            End If
        Next c
    End If
    
    '-Return the index of the last occurrence
    ' or -1 if the item is not found-'
    FindLast = IIf(m, c, -1)
'}
End Function

Public Sub Insert(ByVal Index As Long, ByRef Item As Variant)

'-- Index = The index where to insert the new component --'
'-- Item  = The component to insert --'

'{
    '-Define variables-'
    Dim c As Long
    
    '-The user is not allowed to access reserved items-'
    If (Index >= m_Size) Then
    
        Err.Raise ERR_INDEXOUTOFBOUNDS
        Exit Sub
    End If
    
    '-If there is no free capacity it needs to grow-'
    Do While (m_Capacity < m_Size + 1)
        f_IncreaseCapacity
    Loop
    
    '-Shift each component in the vector with an index
    ' greater or equal to the specified index upward-'
    f_ShiftUpward Index, 1
    
    '-Increase the size of the vector by one-'
    m_Size = m_Size + 1
    
    '-If the item is an object, we have to assign its reference-'
    If IsObject(Item) Then
        Set m_Components(Index) = Item
    Else
        m_Components(Index) = Item
    End If
'}
End Sub

Public Sub InsertArray(ByVal Index As Long, ByRef Items As Variant)

'-- Index     = The index where to insert the new components --'
'-- Items     = The component array to be inserted --'

'{
    'Define variables-'
    Dim c As Long
    
    '-If there is no free capacity it needs to grow-'
    Do While (m_Size + UBound(Items) - LBound(Items) + 1 > m_Capacity)
        f_IncreaseCapacity
    Loop
    
    '-Shift each component in the vector with an index
    ' greater or equal to the specified index upward-'
    f_ShiftUpward Index, UBound(Items) - LBound(Items) + 1
    
    '-Increase the size by the number of element in the array-'
    m_Size = m_Size + UBound(Items) - LBound(Items) + 1
    
    For c = LBound(Items) To UBound(Items)
    
        '-If the item is an object, we have to assign its reference-'
        If IsObject(Items(c)) Then
            Set m_Components(Index + c) = Items(c)
        Else
            m_Components(Index + c) = Items(c)
        End If
    Next c
'}
End Sub

Public Function Items() As Variant()
'{
    '-Exit, if the vector has no components-'
    If (m_Size = 0) Then Exit Function
    
    '-Copy all items...-'
    Items = m_Components
    '-...and size it to the the number of valid components-'
    ReDim Preserve Items(m_Size - 1) As Variant
'}
End Function

Public Sub Remove(ByVal Index As Long, Optional ByVal Value As Long = 1, Optional ByVal HoldPosition As Boolean = False)

'-- Index        = The index of the component to remove --'
'-- Value        = The number of component to remove;
'                  one item is the default --'
'-- HoldPosition = Specifies, whether components with an index greater
'                  than or equal to 'index' are shifted downward --'

'{
    '-Define variables-'
    Dim c As Long
    
    '-The user is not allowed to access reserved items-'
    If (Index >= m_Size) Then
    
        Err.Raise ERR_INDEXOUTOFBOUNDS
        Exit Sub
    End If
    
    If Not HoldPosition Then
        '-Shift each component in the vector with an index
        ' greater or equal to the specified index downward-'
        f_ShiftDownward Index + Value, Value
    Else
    
        '-Set the item(s) to empty-'
        For c = Index To Index + Value - 1
            m_Components(c) = Empty
        Next c
    End If
    
    '-Decrease the size of the vector by the number of removed items-'
    m_Size = m_Size - Value
'}
End Sub

Public Function ToString(ByVal Delimiter As String) As String

'-- Delimiter = The character used to separate the components
'               of the vector in the returned string --'

'{
    '-Define variables-'
    Dim v() As Variant
    
    '-Return a zero length string if the vector has no element-'
    If (m_Size = 0) Then
    
        ToString = ""
        Exit Function
    End If
    
    '-Copy all items to the temporary array...-'
    v = m_Components
    '-...and size it to the number of valid components-'
    ReDim Preserve v(m_Size - 1) As Variant
    
    '-Join each item to the returned string-'
    ToString = Join(v, Delimiter)
    
    '-Release dynamic-array storage space-'
    Erase v
'}
End Function

Public Sub TrimToSize()
'{
    m_Capacity = m_Size
    
    If (m_Capacity = 0) Then
        Erase m_Components
    Else
        ReDim Preserve m_Components(m_Capacity - 1) As Variant
    End If
'}
End Sub
'}

'---Private methods---'

'-- f_IncreaseCapacity = Increases the capacity of the vector by the amount
'                        of the increment --'
'-- f_ShiftUpward      = Shifts each component with an index equal to and
'                        greater than the index upward;
'                        this may affect the size and the capacity --'
'-- f_ShiftDownward    = Shifts each component with an index equal to and
'                        greater than the index downward;
'                        the previous components are discarted --'

'{
Private Sub f_IncreaseCapacity()
'{
    '-If the increment is zero, the capacity of the vector
    ' is doubled, if not, increment is added-'
    If (m_Capacity = 0) And (m_Increment = 0) Then
        m_Capacity = 1
    Else
        m_Capacity = m_Capacity + IIf((m_Increment = 0), m_Capacity, m_Increment)
    End If
    
    ReDim Preserve m_Components(m_Capacity - 1) As Variant
'}
End Sub

Private Sub f_ShiftUpward(ByVal Index As Long, ByVal Value As Long)

'-- Index = The index of the component to start shifting from --'
'-- Value = The amount by which each index is shifted upward --'

'{
    '-Define variables-'
    Dim c As Long
    
    '-Shift the components-'
    For c = m_Size - 1 To Index Step -1
    
        '-If the component is an object, we have to assign its reference-'
        If IsObject(m_Components(c)) Then
            Set m_Components(c + Value) = m_Components(c)
        Else
            m_Components(c + Value) = m_Components(c)
        End If
    Next c
    
    '-Set the now double existing items to empty-'
    For c = Index To Index + Value - 1
        m_Components(c) = Empty
    Next c
'}
End Sub

Private Sub f_ShiftDownward(ByVal Index As Long, ByVal Value As Long)

'-- Index = The index of the component to start shifting from --'
'-- Value = The amount by which each index is shifted downward --'

'{
    '-Define variables-'
    Dim c As Long
    
    '-Shift the components-'
    For c = Index To m_Size - 1
    
        '-If the component is an object, we have to assign its reference-'
        If IsObject(m_Components(c)) Then
            Set m_Components(c - Value) = m_Components(c)
        Else
            m_Components(c - Value) = m_Components(c)
        End If
    Next c
    
    '-Set the last items to empty-'
    For c = m_Size - Value To m_Size - 1
        m_Components(c) = Empty
    Next c
'}
End Sub
'}
